/*****************************************************************************/
/*                             WiNRADiO G66WSB                               */
/*                                SDK DEMO                                   */
/*                      Simple frequency spectrum panel                      */
/*                        (c) Copyright RADIXON 2025                         */
/*****************************************************************************/

#include "u_channelactivity.h"
#include <wx/dcbuffer.h>
#include <math.h>
#include "u_global.h"

IMPLEMENT_DYNAMIC_CLASS(TChannelActivityTuneEvent,wxEvent)

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_CHANNELS_CHANGED,5002)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxEVT_COMMAND_CHANNELS_CHANGED)
DEFINE_EVENT_TYPE(wxEVT_COMMAND_CHANNEL_ACTIVITY_TUNE)

#define EVT_CHANNELS_CHANGED(fn)                                         \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_CHANNELS_CHANGED, -1, -1,    \
	(wxObjectEventFunction)(wxEventFunction)                        \
	(wxNotifyEventFunction) & fn, (wxObject*) NULL ),


BEGIN_EVENT_TABLE(TChannelActivity, wxPanel)
    EVT_PAINT(TChannelActivity::OnPaintEvent)
    EVT_SIZE(TChannelActivity::OnSizeEvent)
    EVT_ERASE_BACKGROUND(TChannelActivity::OnEraseBackgroundEvent)
    EVT_CHANNELS_CHANGED(TChannelActivity::OnData)
    EVT_LEFT_DOWN(TChannelActivity::OnMouseDown)
    EVT_LEFT_UP(TChannelActivity::OnMouseUp)
    EVT_MOTION(TChannelActivity::OnMouseMove)
    EVT_MOUSE_CAPTURE_LOST(TChannelActivity::OnMouseCaptureLost)
END_EVENT_TABLE()

TChannelActivityTuneEvent::TChannelActivityTuneEvent(int Id,uint32_t Channel):wxNotifyEvent(wxEVT_COMMAND_CHANNEL_ACTIVITY_TUNE,Id)
{
    m_Channel=Channel;
}

TChannelActivityTuneEvent::TChannelActivityTuneEvent(const TChannelActivityTuneEvent &Event):wxNotifyEvent(Event)
{
    m_Channel=Event.m_Channel;
}

wxEvent* TChannelActivityTuneEvent::Clone(void) const
{
    return new TChannelActivityTuneEvent(*this);
}

TChannelActivity::TChannelActivity(wxWindow *Parent,wxWindowID Id,const wxPoint &Position,const wxSize &Size):
wxPanel(Parent,Id,Position,Size,wxBORDER_SUNKEN|wxFULL_REPAINT_ON_RESIZE),m_Mutex(wxMUTEX_RECURSIVE)
{
    m_DataChanged=false;
    m_DataValid=false;

    m_MinLevel=-120;

    m_TuneWhenMouseMove=false;

    UpdateParams();
}

TChannelActivity::~TChannelActivity(void)
{

}

void TChannelActivity::UpdateParams(void)
{
 wxClientDC dc(this);
 wxSize Size;
 double Step;
 double x;

    dc.SetFont(GetFont());
    Size=dc.GetTextExtent(wxT("-999"));

    m_GraphRect.SetTopLeft(wxPoint(Size.GetWidth()+2+2,Size.GetHeight()/2+2));
    m_GraphRect.SetBottomRight(wxPoint(GetClientSize().GetWidth()-5,GetClientSize().GetHeight()-Size.GetHeight()-2-2));

    if(m_GraphRect.GetWidth()<10)
    {
        m_GraphRect.SetWidth(10);
    }

    if(m_GraphRect.GetHeight()<10)
    {
        m_GraphRect.SetHeight(10);
    }

    Step=m_GraphRect.GetWidth()/(double)SONOBUOY_CHANNEL_COUNT;

    m_ChannelWidth=(int)Step;
    m_ChannelWidth-=2;

    if(m_ChannelWidth<=0)
    {
        m_ChannelWidth=1;
    }

    x=m_GraphRect.GetLeft()+Step*0.5;

    for(int i=0;i<SONOBUOY_CHANNEL_COUNT;i++,x+=Step)
    {
        m_ChannelX[i]=(int)(x+0.5);
    }

    m_Level2Pixel=(double)(m_GraphRect.GetHeight()-1)/(0-m_MinLevel);
    m_Pixel2Level=1.0/m_Level2Pixel;

    m_LevelStep=5;

    while(m_LevelStep*m_Level2Pixel<=(Size.GetHeight()))
    {
        if(m_LevelStep==5)
        {
            m_LevelStep=10;
        }
        else
        {
            m_LevelStep+=10;
        }
    }
}

void TChannelActivity::OnSizeEvent(wxSizeEvent &Event)
{
    UpdateParams();
    UpdateDisplayData();
}

void TChannelActivity::OnEraseBackgroundEvent(wxEraseEvent &Event)
{
    //does nothing
}

void TChannelActivity::OnPaintEvent(wxPaintEvent &Event)
{
 wxBufferedPaintDC dc(this);
 double fy;
 double Step;
 int y;
 wxString s;
 wxSize Size;
 int Level;

    dc.SetPen(*wxBLACK_PEN);
    dc.SetBrush(*wxBLACK_BRUSH);
    dc.DrawRectangle(wxPoint(0,0),GetClientSize());


    //axis level & freq
    if(IsThisEnabled())
    {
        dc.SetTextForeground(wxColour(0xFF,0xFF,0xFF));
        dc.SetPen(wxPen(wxColour(0xFF,0xFF,0xFF),1));
    }
    else
    {
        dc.SetTextForeground(wxColour(0x80,0x80,0x80));
        dc.SetPen(wxPen(wxColour(0x80,0x80,0x80),1));
    }

    dc.SetTextBackground(wxColour(0,0,0));
    dc.SetFont(GetFont());

    for(int i=4;i<SONOBUOY_CHANNEL_COUNT;i+=5)
    {
        s.sprintf("%d",i+1);
        Size=dc.GetTextExtent(s);
        dc.DrawText(s,m_ChannelX[i]-Size.GetWidth()/2,m_GraphRect.GetBottom()+2);
    }

    s="1";
    Size=dc.GetTextExtent(s);
    dc.DrawText(s,m_ChannelX[0]-Size.GetWidth()/2,m_GraphRect.GetBottom()+2);

    s.sprintf("%d",SONOBUOY_CHANNEL_COUNT);
    Size=dc.GetTextExtent(s);
    dc.DrawText(s,m_ChannelX[SONOBUOY_CHANNEL_COUNT-1]-Size.GetWidth()/2,m_GraphRect.GetBottom()+2);

    //level
    fy=m_GraphRect.GetTop();
    Step=m_LevelStep*m_Level2Pixel;
    Level=0;

    for(;;)
    {
        y=(int)(fy+0.5);

        if(y>m_GraphRect.GetBottom())
        {
            break;
        }

        dc.DrawLine(m_GraphRect.GetLeft(),y,m_GraphRect.GetLeft()-2,y);

        s.sprintf(wxT("%d"),Level);

        Size=dc.GetTextExtent(s);

        dc.DrawText(s,m_GraphRect.GetLeft()-2-Size.GetWidth(),y-Size.GetHeight()/2);

        fy+=Step;
        Level-=m_LevelStep;
    }

    if(IsThisEnabled())
    {
        dc.SetPen(wxPen(wxColour(0xE0,0xE0,0xE0),1,wxSOLID));
    }
    else
    {
        dc.SetPen(wxPen(wxColour(0x70,0x70,0x70),1,wxSOLID));
    }

    dc.DrawLine(m_GraphRect.GetTopLeft(),m_GraphRect.GetBottomLeft());
    dc.DrawLine(m_GraphRect.GetBottomLeft(),m_GraphRect.GetBottomRight());


    if(m_DisplayDataValid && IsThisEnabled())
    {
        dc.SetPen(wxPen(wxColour(0x30,0xE0,0x0),1,wxSOLID));
        dc.SetBrush(wxBrush(wxColour(0x10,0xC0,0x0),wxSOLID));

        for(int i=0;i<SONOBUOY_CHANNEL_COUNT;i++)
        {
            dc.DrawRectangle(m_ChannelRects[i]);
        }
    }

    if(m_Text.Length())
    {
        if(IsThisEnabled())
        {
            dc.SetTextForeground(wxColour(0xFF,0x60,0x30));
        }
        else
        {
            dc.SetTextForeground(wxColour(0x80,0x30,0x20));
        }

        dc.SetTextBackground(wxColour(0,0,0,0));


        Size=dc.GetTextExtent(m_Text);

        dc.DrawText(m_Text,m_GraphRect.GetRight()-Size.GetWidth(),m_GraphRect.GetTop());
    }

    if(m_Text.Length())
    {
        if(IsEnabled())
        {
            dc.SetTextForeground(wxColour(0xFF,0x60,0x30));
        }
        else
        {
            dc.SetTextForeground(wxColour(0x80,0x30,0x20));
        }

        dc.SetTextBackground(wxColour(0,0,0,0));


        Size=dc.GetTextExtent(m_Text);

        dc.DrawText(m_Text,m_GraphRect.GetRight()-Size.GetWidth(),m_GraphRect.GetTop());
    }
}

void TChannelActivity::SetChannels(const float *Channels)
{
    m_Mutex.Lock();

    for(int i=0;i<SONOBUOY_CHANNEL_COUNT;i++)
    {
        m_Channels[i]=Channels[i];
    }

    m_DataValid=true;

    if(!m_DataChanged)
    {
        wxNotifyEvent Event(wxEVT_COMMAND_CHANNELS_CHANGED);

        m_DataChanged=true;

        AddPendingEvent(Event);
    }

    m_Mutex.Unlock();
}

void TChannelActivity::OnData(wxNotifyEvent &Event)
{
    m_Mutex.Lock();

    m_DataChanged=false;

    UpdateDisplayData();

    m_Mutex.Unlock();

    Refresh(false);
}

void TChannelActivity::UpdateDisplayData(void)
{
 uint32_t i;
 int x,h;

    m_Mutex.Lock();

    if(m_DataValid)
    {
        m_DisplayDataValid=true;

        for(i=0;i<SONOBUOY_CHANNEL_COUNT;i++)
        {
            x=m_ChannelX[i]-m_ChannelWidth/2;
            h=(m_Channels[i]-m_MinLevel)*m_Level2Pixel;
            m_ChannelRects[i]=wxRect(x,m_GraphRect.GetBottom()-h,m_ChannelWidth,h);
        }
    }

    m_Mutex.Unlock();
}

bool TChannelActivity::Enable(bool enable)
{
    if(!enable)
    {
        m_TuneWhenMouseMove=false;
    }

    enable=wxPanel::Enable(enable);

    Refresh(false);

    return enable;
}


void TChannelActivity::Clear(void)
{
    m_Mutex.Lock();
    m_DataValid=false;
    m_DataChanged=false;
    m_Mutex.Unlock();

    m_DisplayDataValid=false;
    Refresh(false);
}

void TChannelActivity::OnMouseDown(wxMouseEvent &Event)
{
    m_TuneWhenMouseMove=true;

    CaptureMouse();

    Tune(Event.GetPosition());

    Event.Skip();
}

void TChannelActivity::OnMouseUp(wxMouseEvent &Event)
{
    if(m_TuneWhenMouseMove)
    {
        m_TuneWhenMouseMove=false;
        ReleaseMouse();
    }

    Event.Skip();
}

void TChannelActivity::OnMouseMove(wxMouseEvent &Event)
{
    if(m_TuneWhenMouseMove)
    {
        Tune(Event.GetPosition());
    }

    Event.Skip();
}

void TChannelActivity::Tune(const wxPoint &Pos)
{
    if(m_GraphRect.Contains(Pos))
    {
        for(int i=0;i<SONOBUOY_CHANNEL_COUNT;i++)
        {
            if(m_ChannelRects[i].GetLeft()<=Pos.x && Pos.x<=m_ChannelRects[i].GetRight())
            {
                TChannelActivityTuneEvent Event(GetId(),i+1);
                ProcessEvent(Event);
                break;
            }
        }
    }
}

void TChannelActivity::OnMouseCaptureLost(wxMouseCaptureLostEvent &Event)
{
    m_TuneWhenMouseMove=false;
}

void TChannelActivity::SetText(const wxString &Text)
{
    m_Text=Text;
    Refresh(false);
}
