/*****************************************************************************/
/*                             WiNRADiO G66WSB                               */
/*                                SDK DEMO                                   */
/*                       Simple channel activity panel                       */
/*                        (c) Copyright RADIXON 2025                         */
/*****************************************************************************/

#ifndef __U_CHANNEL_ACTIVITY_H__
#define __U_CHANNEL_ACTIVITY_H__

#include <wx/wx.h>
#include "u_defs.h"
#include "u_types.h"

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_CHANNEL_ACTIVITY_TUNE,5005)
END_DECLARE_EVENT_TYPES()

class TChannelActivityTuneEvent:public wxNotifyEvent
{
public:
    inline uint32_t GetChannel(void) const
    {
        return m_Channel;
    }

    inline void SetChannel(uint32_t Channel)
    {
        if(Channel>=1 && Channel<=99)
        {
            m_Channel=Channel;
        }
    }

    TChannelActivityTuneEvent(const TChannelActivityTuneEvent &Event);
    TChannelActivityTuneEvent(int Id=0,uint32_t Channel=1);

	virtual wxEvent* Clone(void) const;

	DECLARE_DYNAMIC_CLASS_NO_ASSIGN(TChannelActivityTuneEvent)
private:
    uint32_t m_Channel;
};

typedef void (wxEvtHandler::*TChannelActivityTuneEventFunction)(TChannelActivityTuneEvent&);

#define EVT_CHANNEL_ACTIVITY_TUNE(id, fn)                                    \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_CHANNEL_ACTIVITY_TUNE, id, -1,   \
	(wxObjectEventFunction)(wxEventFunction)                         \
	(TChannelActivityTuneEventFunction) & fn, (wxObject*) NULL ),


class TChannelActivity:public wxPanel
{
public:
    void Clear(void);

    void SetText(const wxString &Text);

    void SetChannels(const float *Channels);

    virtual bool Enable(bool enable);

    TChannelActivity(wxWindow *Parent,wxWindowID id,const wxPoint &Position=wxDefaultPosition,const wxSize &Size=wxDefaultSize);
    ~TChannelActivity(void);
protected:
private:
    float               m_Channels[SONOBUOY_CHANNEL_COUNT];
    wxString            m_Text;

    wxRect              m_GraphRect;

    wxRect              m_ChannelRects[SONOBUOY_CHANNEL_COUNT];
    int                 m_ChannelX[SONOBUOY_CHANNEL_COUNT];
    int                 m_ChannelWidth;

    int                 m_MinLevel;

    double              m_Level2Pixel;
    double              m_Pixel2Level;
    int                 m_LevelStep;
    wxMutex             m_Mutex;

    bool                m_DataChanged;
    bool                m_DataValid;

    bool                m_DisplayDataValid;
    wxPoint             *m_DisplayData;
    uint32_t            m_DisplayDataCount;
    bool                m_TuneWhenMouseMove;

    void OnPaintEvent(wxPaintEvent &Event);
    void OnSizeEvent(wxSizeEvent &Event);
    void OnEraseBackgroundEvent(wxEraseEvent &Event);
    void OnData(wxNotifyEvent &Event);
    void OnMouseDown(wxMouseEvent &Event);
    void OnMouseUp(wxMouseEvent &Event);
    void OnMouseMove(wxMouseEvent &Event);
    void OnMouseCaptureLost(wxMouseCaptureLostEvent &Event);


    void UpdateParams(void);
    void UpdateDisplayData(void);
    void Tune(const wxPoint &Pos);

    DECLARE_EVENT_TABLE()
};

#endif //__U_CHANNEL_ACTIVITY_H__
