/*****************************************************************************/
/*                             WiNRADiO G66WSB                               */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2025                         */
/*****************************************************************************/

#ifndef __U_MAINWINDOW_H__
#define __U_MAINWINDOW_H__

#include <wx/setup.h>
#include <wx/wx.h>
#include <wx/splitter.h>
#include <wx/sizer.h>
#include <wx/tglbtn.h>
#include <wx/combobox.h>
#include <atomic>
#include "g66wsbapi.h"
#include "u_spectrum.h"
#include "u_channelactivity.h"
#include "u_intedit.h"
#include "u_waveout.h"

class TMainWindow:public wxFrame,public IG66WSBDeviceCallback
{
public:
    TMainWindow(IG66WSBDevice *Device);
    ~TMainWindow(void);
private:
    IG66WSBDevice           *m_Device;
    G66WSB_DEVICE_INFO      m_DeviceInfo;

    uint32_t                m_DDCCount;

    wxBoxSizer              *m_MainSizer;

    TSpectrum               *m_DDCSpectrum;

    TSpectrum               *m_AudioSpecrum;

    TChannelActivity        *m_ChannelActivity;

    wxStaticText            *m_LAttenuator;
    wxComboBox              *m_CBAttenuator;

    wxToggleButton          *m_BPreamp;

    wxToggleButton          *m_BPower;

    TWaveOut                m_WaveOut;

    wxTimer                 m_DDCSlevelTimer;
    wxTimer                 m_InitTimer;
    wxTimer                 m_StateTimer;

    wxToggleButton          *m_BActiveDDC_Enabled;
    wxComboBox              *m_CBActiveDDC_SonobuoyChannel;
    wxStaticText            *m_LActiveDDC_SLevel;
    wxToggleButton          *m_BActiveDDC_Mute;
    wxStaticText            *m_LActiveDDC_Volume;
    TIntEdit                *m_EActiveDDC_Volume;
    wxStaticText            *m_LActiveDDC_AudioGain;
    TIntEdit                *m_EActiveDDC_AudioGain;
    wxStaticText            *m_LActiveDDC_AudioGain_dB;
    wxStaticText            *m_LAudioSampleRate;
    wxComboBox              *m_CBAudioSampleRate;

    struct TDDCChannel
    {
        uint32_t            m_DDCId;
        wxToggleButton      *m_BEnabled;
        wxToggleButton      *m_BActive;
        wxComboBox          *m_CBSonobuoyChannel;
        wxStaticText        *m_LSLevel;

        std::atomic<float>  m_SLevel;
        std::atomic<float>  m_CompValue;
        uint32_t            m_SonobuoyChannel;
        bool                m_Restart;


        void SetEnabled(bool Enabled)
        {
            m_BEnabled->Enable(Enabled);
            m_CBSonobuoyChannel->Enable(Enabled);
            m_LSLevel->Enable(Enabled);
            m_BActive->Enable(Enabled);
        }
    }                       *m_DDCChannels;

    bool                    m_Connected;

    uint32_t                *m_DDCId2DDCIndex;

    float                   m_DDC_FFT_Coeffs[FFT_SIZE_DDC];
    float                   m_DDC_FFT_Buffer[2*FFT_SIZE_DDC];

    float                   m_Audio_FFT_Coeffs[FFT_SIZE_AUDIO];
    float                   m_Audio_FFT_Buffer[2*FFT_SIZE_AUDIO];

    std::atomic<uint32_t>   m_ActiveDDCIndex;

    uint32_t                m_AudioSampleRate;

    static const wxString   m_AudioSampleRates[];

    virtual void G66WSB_DDCStreamCallback(IG66WSBDevice *Device,uint32_t DDCId,const float *Buffer,uint32_t Count,float Peak,float RMS);
    virtual void G66WSB_AudioStreamCallback(IG66WSBDevice *Device,uint32_t DDCId,uint32_t Stage,const float *Buffer,uint32_t Count);
    virtual void G66WSB_ChannelActivityCallback(IG66WSBDevice *Device,const float *RMS,const float *dBm,uint16_t ADCLevel);

    void OnButtonClick(wxCommandEvent &Event);
    void OnDDCButtonClick(wxCommandEvent &Event);
    void OnActiveDDCButtonClick(wxCommandEvent &Event);
    void OnDDCComboBoxChanged(wxCommandEvent &Event);
    void OnTimer(wxTimerEvent& Event);
    void OnComboBoxChanged(wxCommandEvent &Event);
    void OnChannelActivityTune(TChannelActivityTuneEvent &Event);
    void OnNumericEditChanged(wxNotifyEvent &Event);

    void SetPower(BOOL Power);
    void StartDDC(uint32_t DDCIndex);
    void StopDDC(uint32_t DDCIndex);
    void SetActiveDDC(uint32_t DDCIndex);

    void UpdateState(void);
    void UpdateActiveDDCState(void);
    void SetChannel(uint32_t DDCIndex,uint32_t SonobuoyChannel);
    void UpdateCompValues(void);
    void SetAudioSampleRate(uint32_t Index);

    DECLARE_EVENT_TABLE()
};

#endif

