/* ***********************************************************************/
/*                                                                       */
/*                           WiNRADiO G66WSB                             */
/*      Compat ioctl codes and definitions for G66WSB device driver      */
/*                     (c) Copyright 2025 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G66WSB_IOCTL_COMPAT_H__
#define __G66WSB_IOCTL_COMPAT_H__

#include <linux/compat.h>

#pragma pack(push,1)

struct g66wsb_ioctl_usb_ctrl_vendor_msg_compat
{
    uint8_t         request;
    uint16_t        value;
    uint16_t        index;
    compat_uptr_t   data;
    uint16_t        size;
};

struct g66wsb_ioctl_usb_bulk_msg_compat
{
    uint32_t        ep_index;
    compat_uptr_t   data;
    uint32_t        size;
};

struct g66wsb_ioctl_usb_add_ep_buffer_compat
{
    uint32_t        ep_index;
    compat_uptr_t   buffer;
    uint32_t        size;
};

union g66wsb_ioctl_usb_ep_complete_buffer_compat
{
    uint32_t        ep_index;
    compat_uptr_t   buffer;
};

struct g66wsb_ioctl_pci_dma_queue_buffer_compat
{
    uint16_t        direction;
    uint16_t        channel_index;
    compat_uptr_t   buffer;
    uint32_t        size;
};

union g66wsb_ioctl_pci_dma_complete_buffer_compat
{
    struct g66wsb_ioctl_pci_dma_channel channel;
    compat_uptr_t   buffer;
};

#pragma pack(pop)

#define G66WSB_IOCTL_USB_CTRL_VENDOR_MSG_RCV_COMPAT     _IOW(0x8C,0x17,struct g66wsb_ioctl_usb_ctrl_vendor_msg_compat)
#define G66WSB_IOCTL_USB_CTRL_VENDOR_MSG_SND_COMPAT     _IOW(0x8C,0x18,struct g66wsb_ioctl_usb_ctrl_vendor_msg_compat)
#define G66WSB_IOCTL_USB_BULK_MSG_RCV_COMPAT            _IOW(0x8C,0x19,struct g66wsb_ioctl_usb_bulk_msg_compat)
#define G66WSB_IOCTL_USB_BULK_MSG_SND_COMPAT            _IOW(0x8C,0x1A,struct g66wsb_ioctl_usb_bulk_msg_compat)

#define G66WSB_IOCTL_USB_EP_ADD_BUFFER_COMPAT           _IOW(0x8C,0x33,struct g66wsb_ioctl_usb_add_ep_buffer_compat)
#define G66WSB_IOCTL_USB_EP_GET_COMPLETE_BUFFER_COMPAT  _IOWR(0x8C,0x37,union g66wsb_ioctl_usb_ep_complete_buffer_compat)

#define G66WSB_IOCTL_PCI_DMA_QUEUE_BUFFER_COMPAT        _IOW(0x8C,0x52,struct g66wsb_ioctl_pci_dma_queue_buffer_compat)
#define G66WSB_IOCTL_PCI_DMA_GET_COMPLETE_BUFFER_COMPAT _IOWR(0x8C,0x54,union g66wsb_ioctl_pci_dma_complete_buffer_compat)

#endif // __G66WSB_IOCTL_COMPAT_H__
